#include <stdint.h>
#include <compat/console.h>
#include <compat/usleep.h>

int console_quit = 0;

#ifdef WIN32
#include <conio.h>

void console_init(void) {
}

void console_fini(void) {
}

int console_getc(int timeout_ms) {
    if (timeout_ms == -1)
        return _getch();

    while (timeout_ms--) {
        if (_kbhit())
            return _getch();
        usleep(1000);
    }
    return CONSOLE_TIMEOUT;
}

void console_putc(uint8_t c) {
    _putch(c);
}

#else

#include <signal.h>
#include <termios.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <compat/serial.h>

static struct termios t, t_stdin , t_stdout;

static void handler(int sig) {
    console_quit = 1;
}

void console_init(void) {
    signal(SIGINT, handler);

    if (isatty(STDIN_FILENO)) {
        tcgetattr(STDIN_FILENO, &t_stdin);
        tcgetattr(STDIN_FILENO, &t);
        t.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
        t.c_lflag &= ~(ECHO|ECHONL|ICANON|IEXTEN);
        t.c_cflag &= ~(CSIZE|PARENB);
        t.c_cflag |= CS8;
        tcsetattr(STDIN_FILENO, TCSANOW, &t);
    }

    if (isatty(STDOUT_FILENO)) {
        tcgetattr(STDOUT_FILENO, &t_stdout);
        tcgetattr(STDOUT_FILENO, &t);
        t.c_oflag &= ~OPOST;
        t.c_lflag &= ~(ECHO|ECHONL|ICANON|IEXTEN);
        t.c_cflag &= ~(CSIZE|PARENB);
        t.c_cflag |= CS8;
        tcsetattr(STDOUT_FILENO, TCSANOW, &t);
    }
}

void console_fini(void) {
    tcsetattr(STDOUT_FILENO, TCSANOW, &t_stdout);
    tcsetattr(STDIN_FILENO, TCSANOW, &t_stdin);
}

int console_getc(int timeout_ms) {
    return serial_getc(STDIN_FILENO, timeout_ms);
}

void console_putc(uint8_t c) {
    serial_putc(STDOUT_FILENO, c);
}

#endif

void console_put(void *buf, int len) {
    uint8_t *c = buf;
    while (len--)
        console_putc(*c++);
}
